	page	60,132

        Title   LCD_UF.COM lcd driver
        Subttl  Copyright JK microsystems 1997-2004

;************************************************************************;
;*								       	*;
;*      Copyright JK microsystems 1997-2002     -  All Rights Reserved          *;
;*									*;
;*	This software is NOT shareware, freeware, or public domain.	*;
;*	It is the property of JK microsystems.				*;
;*									*;
;*	Customers of JK microsystems may modify the source code		*; 
;*	and/or distribute the binary image of this software without 	*;
;*	additional costs provided it is run only on hardware 		*;
;*	manufactured by JK microsystems.  All other use is expressly 	*;
;*	prohibited.							*;
;*									*;
;*	Update Log							*;
;*									*;
;*      Version  Date        Comments                     Progammer     *;
;*      ------- ------- ------------------------------- -------------   *;
;*      1.0     7-16-95     First release                       jds     *;
;*      2.0     5-18-96     Corrected timing error                      *;
;*                          Now works with Seiko LCDs           jds     *;
;*      3.0     12-17-97    Version for 386ex                   jds     *;
;*      3.1     5-19-99     Fix for reset with Winbond lcd      jds/Rob Peter *;
;*      3.2     10-2-01     Copy init code from LCD_LF.                 *;
;*                          Should resolve warm boot probs.     ew      *;
;*      4.0     8-20-02     Version for 186                     ew      *;
;*	5.0	6-9-04	    Version for uFlash			jds	*;	
;************************************************************************;

Code_seg	Segment
	assume	cs:code_seg,ds:code_seg,ss:code_seg

	org	100h
start:					; Put the initialization code at
	jmp	init			; the end so we don't have to save it
	
;-----------------------------------------------------------------------------
;
;	This is invoked with each bios LPT interrupt
;
LPT_int:
	sti				; interrupts back on
	cmp	dx,0			; LPT number greater than 1
	je	do_lcd			; no, do it
	jmp	chain			; yes, chain to old handler

do_lcd:
	push	dx			; save a couple registers
	push	bx

	cmp	al,0A0h			; command escape char?
	jne	LPT1_not_esc		; no, go around
	mov	cs:[LPT1_cmd_flag],1	; yes, set the flag
	jmp	lpt_done		; and return

LPT1_not_esc:
	mov	bl,al			; save the char
	shr	al,1			; move the first nibble over	
	shr	al,1
	shr	al,1
	shr	al,1
	cmp	cs:[lpt1_cmd_flag],1	; was it a command?
	je	send_LPT1_cmd		; yes, go elsewhere to send it
	call	data_write		; no, it's data so send it here
	mov	al,bl			; restore the data
	call	data_write		; send the low nibble
	jmp	lpt_done		; and leave

send_LPT1_cmd:
	call	cmd_write		; come here to send it as a command
	mov	al,bl			; restore the data
	call	cmd_write		; and send the low nibble
	mov	cs:[lpt1_cmd_flag],0	; clear the flag
LPT_done:
	mov	ah,0D0h			; return good status
	pop	bx
	pop	dx			
	iret

chain:
	push	cs:[old_LPT_seg]	; pushing these and leaving with
	push	cs:[old_LPT_off]	; a retf will chain to old lpt
	retf				; routine


;----------------------------------------------------------------------------
;
;	Delay for 5ms
;
delay_5_ms:
	push 	ax			; ax gets trashed
	mov 	ax,1000			; time constant for 5ms

delay_5_ms_loop:
	dec	ax			; do the loop
	jnz	delay_5_ms_loop
	pop	ax
	ret
		
;----------------------------------------------------------------------------
;
;	Set the bus to idle state
;
bus_idle:
	push	ax
	push	dx
        mov     dx,port3out 		; port3
	in	al,dx			; must preserve upper 6 bits
	and	al,0F8h			; zero bits 3 to 0
	or	al,1			; idle state
	out	dx,al			; do it
	call	delay_5_ms	
	pop	dx
	pop	ax
	ret

;----------------------------------------------------------------------------
;
;	Short Delay
;
short_delay:
	push	ax
	pop	ax
	ret

;----------------------------------------------------------------------------
;
;	Write a command to the lcd
;
cmd_write:
	shl	al,1			; have to align nibble with
	shl	al,1			; upper 4 bits
	shl	al,1
	shl	al,1

	and	al,0F0h			; zero bottom nibble
	mov	ah,al			; save bus data to ah

        mov     dx,port1out        	; point to lcd data bus
	in	al,dx			; get port value
	and	al,0Fh			; zero top nibble
	or	al,ah			; OR in bus value
	out	dx,al			; put AL 0-3 on the bus

        mov     dx,port3out          	; get port3
	in	al,dx
	and	al,0F8h			; RS=true, WR=true, E=false
	out	dx,al			; send it
	call	short_delay

	or	al,4			; raise E to make it true
	out	dx,al			
	call	short_delay

	and	al,0F8h			; drop E
	out	dx,al
	call	short_delay

	call	bus_idle		; restore the bus to idle
	call	delay_5_ms		; wait for lcd to do it
	ret

;----------------------------------------------------------------------------
;
;	Write data to the lcd specified in ah
;
data_write:
	shl	al,1			; have to align nibble with
	shl	al,1			; upper 4 bits
	shl	al,1
	shl	al,1

	and	al,0F0h			; zero bottom nibble
	mov	ah,al			; save bus data to ah

        mov     dx,port1out        	; point to lcd data bus
	in	al,dx			; get port value
	and	al,0Fh			; zero top nibble
	or	al,ah			; OR in bus value
	out	dx,al			; put AL 0-3 on the bus

        mov     dx,port3out         
	in	al,dx
	and	al,0F8h
	or	al,2			; drop the WR/ line, raise RS
	out	dx,al			; send it
	call	short_delay

	or	al,4
	out	dx,al			; raise E0 or E1
	call	short_delay
	and	al,0FBh			; drop E0 and E1
	out	dx,al
	call	short_delay
	call	bus_idle

	ret

;-----------------------------------------------------------------------------
;
;	Data and equates	
;
old_LPT_seg	dw	0		; save the old LPT routine
old_LPT_off	dw	0		; vectors in case we need them

LPT1_cmd_flag	db	0		; 1 if next byte is a command

port1dir	equ	word ptr 0F864h	; port 1, direction
port1out 	equ	word ptr 0F862h	; port 1, bits 0-3 used for LCD data
port3dir	equ	word ptr 0F874h	; port 3, direction
port3out	equ	word ptr 0F872h	; port 3, LCD control signals

last_add:				; last address to save when terminating

;-----------------------------------------------------------------------------
;
;	This is our initialization code
;
init:

	mov	ax,40h			; point to bios data area
	mov	ds,ax

	mov	ax,ds:[10h]		; get equipment word
	or	ah,010000000B		; bits 14 and 15 = number of printers
	and	ah,001111111B		; clear bit 15, set bit 14, 1 printer
	mov	ds:[10h],ax		; put equipment word back

	mov	ax,0			; point to interrupt vector table
	mov 	ds,ax

	cli				; no ints while changing vectors

	mov	ax,ds:[5Ch]		; get int 17h offset
	mov	cs:[old_LPT_off],ax	; save it
	mov	ds:[5Ch],offset LPT_int ; and replace it with our routine
	mov	ax,ds:[5Eh]		; get int 17h segment
	mov	cs:[old_LPT_seg],ax	; save it
	mov	ax,cs			; get our segment
	mov	ds:[5Eh],ax		; and install it

	sti				; ints back on

	mov	dx,port3dir		; set the direction of port3
	in	al,dx			; bits 2  1  0 
	and	al,0F8h			;      E  RS R/W
	out	dx,al			; make them all outs

	mov	dx,port1dir		; set the direction of port1
	in	al,dx			; bits 7,6,5,4 = LCD data 3,2,1,0
	and	al,0Fh
	out	dx,al

	call	bus_idle		; set bus_idle state

; ---------- LCD init code starts here

	mov	cx,3			; send 3 03 commands to lcd 0
init_loop0:
	call	delay_5_ms
	mov	ah,0
	mov	al,3
	call	cmd_write
	loop	init_loop0

	call	delay_5_ms		; give it some time to settle
	mov	ah,0			; lcd 0
	mov	al,2			; set to 4 bit bus
	call	cmd_write		; write it

	mov	cx,8			; init string is 8 bytes
	mov	di,offset init_string	; clear the lcd and home the cursor
clear_loop0:
	mov	dx,0			; lcd 0
	mov	al,cs:[di]
	int	17h
	inc	di
	loop 	clear_loop0

; ---------- LCD init code ends here

tsr:
	mov	ax,3100h		; TSR function code
	mov 	dx,offset last_add 	; get last address used
	mov 	cl,4			; convert to paragraphs
	shr     dx,cl			; by dividing by 16
	inc	dx			; and rounding up by one

	int	21h			; tell dos to do it

init_string:
	db	160,40			; 2 lines
 	db	160,12			; display on
	db	160,6			; position increment, no shift
	db	160,1			; clear screen, home cursor


code_seg 	ends

	end	start
